/**
  *  \file arch\win32-bcc55\shell.cc
  *  \brief Shell folders etc.
  */

#define CINTERFACE
#include <shlobj.h>
#include <windows.h>
#include "arch/platform.h"
#include "cpluslib/string.h"
#include "util/console.h"
#include "arch/win32/win32.h"

#if 1
static LPMALLOC
getShellAllocator()
{
    static LPMALLOC m = 0;
    if (!m)
        SHGetMalloc(&m);
    return m;
}

static string_t
getShellFolder(int csidl)
{
    LPITEMIDLIST il;
    char buffer[MAX_PATH];

    if (!SUCCEEDED(SHGetSpecialFolderLocation(NULL, csidl, &il)))
        return string_t();

    /* Windows 95 doesn't have SHGetPathFromIDListW. It doesn't even
       have a stub in its shell32.dll, so that dynamic linking fails.
       We therefore always use the ANSI version, under the assumption
       that the well-known folders are always stored using names
       representable in the ANSI codepage. */
    BOOL rv = SHGetPathFromIDList(il, buffer);
    if (LPMALLOC m = getShellAllocator())
        m->lpVtbl->Free(m, il);
    if (rv)
        return win32::convertFromANSI(buffer);
    else
        return string_t();
}

string_t
Platform::getUserSettingsDirectory()
{
    string_t rv = getShellFolder(CSIDL_APPDATA);
    if (rv.length())
        return rv;
    else
        return string_t("C:\\");
}
#else
string_t
Platform::getUserSettingsDirectory()
{
    char c;
    UINT size = GetWindowsDirectory(&c, 0);
    if (size) {
        char* n = new char[size+1];
        GetWindowsDirectory(n, size+1);
        string_t rv(n);
        delete[] n;
        return rv;
    } else {
        return string_t("C:\\");
    }
}
#endif
