/**
  *  \file client/wtile.h
  *
  *  (c) 2001-2011 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_CLIENT_WTILE_H
#define PCC_V2_CLIENT_WTILE_H

#include "ui/ui.h"
#include "gfx/gfx.h"
#include "ui/skin.h"
#include "game/objtypes.h"

class WTile : public UIBaseComplexWidget {
    GfxPoint size;
    string_t heading;
    int      y_origin;
    UIGroup  children;
 public:
    enum State {
        sNormal,
        sHeadingOnly,
        sContentOnly
    };
    enum Layout {
        LeftJustified,
        RightJustified
    };
 private:
    State state   : 8;
    Layout layout : 8;
    int lrmargin;
    bool children_move;

    void computeCoordinates();
 protected:
    virtual void drawData(GfxCanvas& can) = 0;

    void setLayout(Layout layout, int lrmargin, bool children_move);
    
    void addControl(GfxRect r, UIBaseWidget& w);

    void onResize(GfxRect oldsize, bool by_layout = false);

 public:
    WTile(GfxPoint size, int aid = 0);
    ~WTile();

    void drawContent(GfxCanvas& can);
    void getLayoutInfo(LayoutInfo& info);

    void setDisplayState(State st);
    State getDisplayState() const
        { return state; }

    WTile& setHeading(string_t heading);
    const string_t getHeading() const
        { return heading; }

    GfxRect getInnerSize(int lmargin, int rmargin);
};

class WObjectTypeObserverTile : public WTile,
                                public GObjectTypeObserver {
 public:
    WObjectTypeObserverTile(GfxPoint size, GObjectType& type);
    ~WObjectTypeObserverTile();
    void onObjectChanged();
};

#endif
