/**
  *  \file game/prefio.cc
  *
  *  FIXME: having this in a separate module means that checkinits.pl
  *  will not warn of a missing mod_pref.
  *
  *  (c) 2001-2011 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "game/pref.h"
#include "arch/platform.h"
#include "cpluslib/format.h"
#include "io/dirs.h"
#include "io/memstream.h"
#include "io/streams.h"
#include "util/confpars.h"
#include "util/console.h"
#include "util/modinit.h"
#include "util/nls.h"

static string_t
getPrefFileName()
{
    return Directory::makeFileName(Platform::getUserSettingsDirectory(), "pcc2.ini");
}

/** Load user preferences file (pcc2.ini). */
static void
loadUserPreferences()
{
    Ptr<Stream> s = Stream::openFileNT(getPrefFileName(), Stream::C_READ);
    if (s) {
        ConfigFileParser cfp(getUserPreferences(), true);
        cfp.setUtf8();
        cfp.parseFile(*s);
    }
}

/** Save user preferences file (pcc2.ini). */
static void
saveUserPreferences()
{
    string_t name = getPrefFileName();

    // Update config file in memory
    MemoryStream ms;
    ConfigFileUpdater cfu(getUserPreferences(), ms);
    if (Ptr<Stream> s = Stream::openFileNT(name, Stream::C_READ)) {
        cfu.parseFile(*s);
    }
    cfu.finish();

    // Write it out
    Ptr<Stream> s = Stream::openFileNT(name, Stream::C_CREATE | Stream::C_WRITE);
    if (!s) {
        console.write(LOG_ERROR, format(_("Unable to create configuration file \"%s\""), name));
    } else {
        s->writeT(ms.getBuffer().ptr(), ms.getBuffer().getSize());
    }
}

extern const ModuleInitializer mod_pref[] = {
    { OnEnterNoGame | OnEnterDirectory, loadUserPreferences },
    { OnLeftDirectory,                  saveUserPreferences },
    { 0, 0 },
};
