/**
  *  \file int/array.h
  */
#ifndef PCC_V2_INT_ARRAY_H
#define PCC_V2_INT_ARRAY_H

#include <vector>
#include "int/value.h"
#include "int/compound.h"
#include "int/data.h"

/** Storage for array data. Consists of a set of dimensions, plus an
    IntDataSegment containing the array data. */
class IntArrayData {
    /** Total size. Used to keep track of the maximum total number of elements. */
    uint32_t total_size;

    /** Dimensions of the array. */
    std::vector<uint32_t> dimensions;

 public:
    IntArrayData();
    ~IntArrayData();

    bool addDimension(int32_t n);
    bool computeIndex(IntArgBlock& args, uint32_t& result) const;

    uint32_t getNumDimensions() const;
    uint32_t getDimension(uint32_t i) const;

    IntDataSegment content;
};

/** Array reference. Arrays are always by-reference, because IntArray
    objects are cloned when put on the stack. The actual data is
    stored in an IntArrayData object. */
class IntArray : public IntIndexableValue {
    Ptr<IntArrayData> data;

    bool computeIndex(IntArgBlock& args, uint32_t& result);
 public:
    IntArray(Ptr<IntArrayData> data);

    // IntIndexableValue:
    IntValue* get(IntArgBlock& args);
    void set(IntArgBlock& args, IntValue* value);
    IntContext* makeFirstContext();

    // IntValue:
    string_t    toString(bool readable);
    void        store(IntTagNode&, Stream&);
    IntValue*   clone();

    // Inquiry
    IntArrayData& getData();
};

inline uint32_t
IntArrayData::getNumDimensions() const
{
    return dimensions.size();
}

inline uint32_t
IntArrayData::getDimension(uint32_t i) const
{
    return dimensions[i];
}

inline IntArrayData&
IntArray::getData()
{
    return *data;
}

#endif
