/**
  *  \file reshack/colors.h
  *  \brief Resource Editor: Colors
  *
  *  (c) 2001-2011 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_RESHACK_COLORS_H
#define PCC_V2_RESHACK_COLORS_H

#include "cpluslib/smartptr.h"
#include "gfx/pixmap.h"

/** Color mode. */
enum ColorMode {
    /** Image uses standard palette colors. That is, the first
        COLOR_AVAIL colors equal our palette and are not modifyable,
        the following colors are variable. */
    StandardPaletteColor,
    /** Image uses palette colors, but those differ from our standard
        palette. */
    ForeignPaletteColor,
    /** Image uses grayscales (for font editing, never auto-detected).
        Internally, this is a regular 8-bpp standard palette image. */
    GrayscaleColor,
    /** Image uses true color, i.e. no palette. */
    TrueColor
};

ColorMode identifyColorMode(Ptr<GfxPixmap> pix);
Ptr<GfxPixmap> makeEditable(Ptr<GfxPixmap> pix);
bool isEditableColor(ColorMode cm, uint8_t color);
void setScreenPalette(Ptr<GfxPixmap> pix);
uint32_t colorIndexToScreenColor(Ptr<GfxPixmap> pix, uint8_t index);
uint32_t colorIndexToImageColor(Ptr<GfxPixmap> pix, uint8_t index);

#endif
