/**
  *  TEST: UTILx.DAT reader
  */

#include <iostream>
#include "io/streams.h"
#include "game/univ.h"
#include "game/utilparse.h"
#include "arch/platform.h"
#include "util/modinit.h"

class MyUtilReader : public GUtilReader {
    GUniverse& univ;
 public:
    MyUtilReader(GUniverse& univ)
        : GUtilReader(),
          univ(univ)
        { }

    bool process(int rec_id, std::size_t size, const char data[]);
    void failure(Stream& s);
};

bool
MyUtilReader::process(int rec_id, std::size_t size, const char data[])
{
    std::cout << "Record " << rec_id << ", " << size << " bytes\n";

    ptr_vector<GMessageInformation> info;
    GUtilMessageParser(univ).parseRecord(rec_id, size, data, info);

    for (unsigned int i = 0; i < info.size(); ++i) {
        GMessageInformation& mi = *info[i];
        switch (mi.getObjectType()) {
         case MessageShip: std::cout << "- Ship " << mi.getObjectId() << "\n"; break;
         case MessageMinefield: std::cout << "- Minefield " << mi.getObjectId() << "\n"; break;
         case MessagePlanet: std::cout << "- Planet " << mi.getObjectId() << "\n"; break;
         case MessageNoObject: std::cout << "- No associated object\n"; break;
        }
        for (GMessageInformation::iterator j = mi.begin(); j != mi.end(); ++j) {
            if (GMessageStringValue* sv = dynamic_cast<GMessageStringValue*>(*j)) {
                std::cout << "  . " << getNameForItem(sv->getIndex()) << ": '" << sv->getValue() << "'\n";
            } else if (GMessageIntValue* iv = dynamic_cast<GMessageIntValue*>(*j)) {
                std::cout << "  . " << getNameForItem(iv->getIndex()) << ": " << iv->getValue() << "\n";
            } else {
                std::cout << "  . (unknown information)\n";
            }
        }
    }
    return true;
}

void
MyUtilReader::failure(Stream& s)
{
    // same message as FileTooShortException
    std::cout << s.getName() << ": " << "File is too short\n";
}

int main(int argc, char** argv)
{
    performInitialisation(OnStartup, mod_base_console);
    argv = Platform::convertArgumentsToUtf8(argv);

    GUniverse someUniverse;
    someUniverse.init();

    while (*++argv) {
        Ptr<Stream> s = Stream::openFileNT(*argv, Stream::C_READ);
        if (!s) {
            std::cout << "Cannot open " << *argv << "\n";
        } else {
            std::cout << "=== " << *argv << " ===\n";
            MyUtilReader(someUniverse).read(*s);
        }
    }
    return 0;
}
