/**
  *  \file ui/layout-null.cc
  *
  *  (c) 2001-2011 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "ui/layout.h"

/*! \class UINullLayout
    Null layout. Does not implement deeper layout logic; instead, the container
    will be treated as a fixed-size widget that can just move around with its
    content. */

UINullLayout::UINullLayout()
{ }

void
UINullLayout::doLayout(UILayoutableWidget& container, GfxRect old_size, GfxRect new_size)
{
    // Find move offset
    int dx = new_size.x - old_size.x;
    int dy = new_size.y - old_size.y;

    // Move all children
    for (UIBaseWidget* w = container.getFirstChild(); w != 0; w = w->getNextSibling()) {
        UIBaseWidget::LayoutInfo winfo;
        w->getLayoutInfo(winfo);
        if ((winfo.flags & UIBaseWidget::li_NoLayout) == 0) {
            GfxRect pos = w->getExtent();
            pos.moveBy(dx, dy);
            w->setExtent(pos, true);
        }
    }
}

void
UINullLayout::getLayoutInfo(UILayoutableWidget& container, UIBaseWidget::LayoutInfo& info)
{
    info.min_size = info.pref_size = container.getExtent().getSize();
    info.flags = 0;
}

UINullLayout UINullLayout::instance;
