/**
  *  \file util/answer.h
  *  \brief AnswerProvider interface
  *
  *  (c) 2001-2011 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_UTIL_ANSWER_H
#define PCC_V2_UTIL_ANSWER_H

#include "cpluslib/object.h"
#include "cpluslib/string.h"

/** Interface to provide answers to questions. */
class AnswerProvider : public Object {
 public:
    /** Answer a question.
        \param questionId Identifier of the question, program interface
        \param question   Question, user interface */
    virtual bool ask(int questionId, string_t question) = 0;
};

/** Implementation of AnswerProvider to provide constant answers. */
class ConstantAnswerProvider : public AnswerProvider {
    const bool answer;
 public:
    ConstantAnswerProvider(bool answer)
        : answer(answer)
        { }

    bool ask(int, string_t);

    /** Constant answer provider that always answers affirmative. */
    static ConstantAnswerProvider sayYes;

    /** Constant answer provider that always answers negative. */
    static ConstantAnswerProvider sayNo;
};

#endif
